#ifndef CHIERROR_H
#define CHIERROR_H

/****************************************************************************
** COPYRIGHT (C):    1994 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          ChiLib
** FILE:             chierror.h
** PURPOSE:          Error handling
** VERSION           1.0
** PROGRAMMER:       Cay Horstmann (CSH)
** START DATE:       4-26-94 (CSH)
** UPDATE HISTORY:
****************************************************************************/

#line 14 "chierror.cpp"

#include "chisetup.h"

#line 18 "chierror.cpp"

#include <iostream.h>

#line 26 "chierror.cpp"


class Chi_Error
/* PURPOSE:    report an error
*/
{
public:
   Chi_Error(const char* f, int l, const char* expr = 0, const char* msg = 0);
   void abort();
   virtual void message(ostream&) const;

private:
   const char* _msg;
   const char* _expr;
   const char* _file;
   int _line;
};

/*-------------------------------------------------------------------------*/

class Chi_BoundsError : public Chi_Error
/* PURPOSE:    report an array or string bounds error
*/
{
public:
   Chi_BoundsError(const char* f, int l, int i, int lo, int hi);
   virtual void message(ostream&) const;
private:
   int _actual;
   int _low;
   int _high;
};

/*-------------------------------------------------------------------------*/

class Chi_PrecondError : public Chi_Error
/* PURPOSE:    report a precondition failure
*/
{
public:
   Chi_PrecondError(const char* f, int l, const char* expr);
};

/*-------------------------------------------------------------------------*/

class Chi_PostcondError : public Chi_Error
/* PURPOSE:    report a postondition failure
*/
{
public:
   Chi_PostcondError(const char* f, int l, const char* expr);
};

/*-------------------------------------------------------------------------*/

class Chi_InvariantError : public Chi_Error
/* PURPOSE:    report an invariant failure
*/

{
public:
   Chi_InvariantError(const char* f, int l, const char* expr);
};

/*-------------------------------------------------------------------------*/

class Chi_MemError : public Chi_Error
/* PURPOSE:    report memory exhaustion
*/
{
public:
   Chi_MemError(const char* f, int l, const char* expr);
};


#line 104 "chierror.cpp"


#ifdef CHI_THROW_ERRORS
#define CHI_THROW(E) (throw E)
#else
#define CHI_THROW(E) (E.abort())
#endif

#define CHI_ASSERT(E) ((E) ? (void)0 : CHI_THROW(Chi_Error(__FILE__, __LINE__, #E)))
#define CHI_ASSERT_PRECOND(E) ((E) ? (void)0 : CHI_THROW(Chi_PrecondError(__FILE__, __LINE__, #E)))
#define CHI_ASSERT_POSTCOND(E) ((E) ? (void)0 : CHI_THROW(Chi_PostcondError(__FILE__, __LINE__, #E)))
#define CHI_ASSERT_INVARIANT(E) ((E) ? (void)0 : CHI_THROW(Chi_InvariantError(__FILE__, __LINE__, #E)))
#define CHI_ASSERT_MEM(E) ((E) ? (void)0 : CHI_THROW(Chi_MemError(__FILE__, __LINE__, #E)))
#define CHI_ASSERT_BOUNDS(I, L, H) (chi_check_bounds(__FILE__, __LINE__, I, L, H))


#line 121 "chierror.cpp"
extern void chi_check_bounds(const char* f, int l, int i, int lo, int hi);

#endif
