/****************************************************************************
** COPYRIGHT (C):    1994 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          ChiLib
** FILE:             chigc.cpp
** PURPOSE:          select graphics context include file
** VERSION           1.0
** PROGRAMMER:       Cay Horstmann (CSH)
** START DATE:       7-29-94 (CSH)
** UPDATE HISTORY:
****************************************************************************/

#ifndef EXPORT

#if defined(CHI_BGI)
#include "chigcbgi.h"
#elif defined(CHI_WIN)
#include "chigcwin.h"
#elif defined(CHI_X11)
#include "chigcx11.h"
#elif defined(CHI_ANY)
#include "chistrng.h"

class Chi_MouseEvent;

class Chi_GraphicsContext
/* PURPOSE: public interface for graphics context only--needed for library
            building
   REMARKS: do not use this class in your code--always #define
            CHI_WIN, CHI_BGI, ...
*/
{
public:
   enum Color
   {  BLACK, BLUE, GREEN, CYAN,
      RED, MAGENTA, BROWN, LIGHTGRAY, /* dark colors */
      DARKGRAY, LIGHTBLUE, LIGHTGREEN, LIGHTCYAN,
      LIGHTRED, LIGHTMAGENTA, YELLOW, WHITE /* light colors */,
      DEFAULT_COLOR = BLACK
         /* default color for drawing--opposite from background */
   };
   enum PenStyle { NORMAL_PEN, THICK_PEN, DOTTED_PEN, DASHED_PEN };
   enum BrushStyle { EMPTY_BRUSH, SOLID_BRUSH,
      HORIZ_BRUSH, FDIAG_BRUSH, BDIAG_BRUSH, XDIAG_BRUSH };
   enum FontStyle { SYS_FONT, SANS_FONT, ROMAN_FONT };
   enum RopMode { COPY_ROP, XOR_ROP, AND_ROP, NOT_ROP, OR_ROP };

   void close(Bool wait = TRUE);

   void point(double x, double y);
   void line(double xfrom, double yfrom, double xto, double yto);
   void arc(double x, double y, double ra, double rb, double x1, double y1,
      double x2, double y2);
   void rectangle(double xleft, double ytop, double xright, double ybot);
   void ellipse(double x, double y, double ra, double rb);
   void piesegment(double x, double y, double ra, double rb, double x1, double y1,
      double x2, double y2);

   void text(Chi_String t, double x, double y);
   void text_extent(Chi_String t, double& x, double& y);

   void set_brush_style(BrushStyle);
   void set_brush_color(Color);
   void set_pen_style(PenStyle);
   void set_pen_color(Color);
   void set_font_size(double s);
   void set_font_style(FontStyle);
   void set_rop_mode(RopMode);

   void set_window_coord(double xmin, double ymin, double xmax, double ymax);
   void set_clip_rectangle(double xmin, double ymin, double xmax, double ymax);
   void clear();
   void mouse_events(int (*h)(const Chi_MouseEvent&, void*), void* p);
private: // to avoid instantiation
   Chi_GraphicsContext();
};

#else
#error Define CHI_BGI, CHI_WIN, ...
#endif
#endif
