/****************************************************************************
** COPYRIGHT (C):    1994 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          Mastering Object-Oriented Design
** FILE:             chigcx.cpp
** PURPOSE:          Graphics context class (X-Window's version)
** VERSION           1.0
** COMPILER:         Sun CC
** TARGET:           Sun SparcStation
** PROGRAMMER:       Cay Horstmann (CSH)
**		     Charles Miller (CBM)
**                   Darrel D. Cherry (DDC)
** START DATE:       6-5-94   (CBM)
** NOTES:
	  Because X does not have a standard font system (such as those of
	  MS-Windows and Borland's BGI), the programmer must decide what
	  font to use for "system," "sans," and "roman" selections.  Also,
	  because scaling is not provided on all systems, this version of the
	  Graphics Context class does not allow for a changing of the font
	  sizes; here it is a stub.
** UPDATE HISTORY:  1-13-96 (DDC) 
          Font scaling has been implemented. There are new variables 
          and member functions in the Chi_GraphicsContext class for font
          scaling, and they are labeled as such.

****************************************************************************/

/* IMPORT ******************************************************************/
												   

#include <math.h>
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>

#include "chigcx11.h" // contains X include files

EXPORT
#include "chisetup.h"

EXPORT
#include "chistrng.h"


#ifndef EXPORT

EXPORT
#include <X11/Xlib.h>

EXPORT
#include <X11/Xutil.h>

EXPORT
#include <X11/Xos.h>

EXPORT
#include <X11/Xatom.h>

#endif


//** Icon bitmap **
#define icon_bitmap_width 40
#define icon_bitmap_height 40
static char icon_bitmap_bits[] = {
   0xc3, 0xc3, 0x7f, 0x00, 0x78, 0x00, 0x00, 0x00, 0x00, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x38, 0x00, 0x40, 0x00, 0x80, 0x24, 0x00, 0x00, 0x00,
   0x80, 0x44, 0x00, 0x00, 0x00, 0x80, 0x44, 0x00, 0x00, 0x00, 0x80, 0x74,
   0x00, 0x0f, 0x0c, 0x00, 0x7c, 0x3e, 0x41, 0x0e, 0x00, 0x44, 0x22, 0x41,
   0x02, 0x00, 0x84, 0x22, 0x46, 0x02, 0x00, 0x9c, 0x26, 0xcc, 0x02, 0x00,
   0x78, 0x3c, 0xcd, 0x36, 0x80, 0x00, 0x20, 0x06, 0x0c, 0x80, 0x01, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x02, 0x40, 0x00, 0x80, 0x01, 0x06, 0x40, 0x00,
   0x80, 0x01, 0x04, 0x20, 0x00, 0x80, 0x01, 0x04, 0x20, 0x01, 0x80, 0x01,
   0x04, 0x20, 0x00, 0x80, 0x01, 0x04, 0x22, 0x00, 0x80, 0x01, 0x04, 0x33,
   0xf1, 0x81, 0x01, 0x88, 0x12, 0x31, 0x03, 0x01, 0x88, 0x12, 0x11, 0x02,
   0x00, 0x88, 0x12, 0x11, 0x02, 0x00, 0x48, 0x1a, 0x11, 0x02, 0x00, 0x70,
   0x04, 0x19, 0x82, 0x01, 0x00, 0x00, 0x00, 0x80, 0x01, 0x00, 0x00, 0x38,
   0x80, 0x01, 0x00, 0x00, 0xce, 0x80, 0x01, 0x00, 0x00, 0x83, 0x81, 0x81,
   0x07, 0x80, 0x01, 0x81, 0xe1, 0x04, 0xc0, 0x00, 0x83, 0x31, 0x08, 0x40,
   0x00, 0x82, 0x10, 0x08, 0x20, 0x00, 0x82, 0x19, 0x10, 0x30, 0x00, 0x86,
   0x0c, 0x30, 0x18, 0x00, 0x84, 0x04, 0x60, 0x0e, 0x00, 0xdc, 0x02, 0x80,
   0x03, 0x00, 0x70, 0x00, 0x00, 0x00, 0x00, 0x00};


// The following are some aliases for various X fonts.
// Make sure these are available on your system.  Substitute as necessary.
#define SANS_14	"-adobe-helvetica-bold-r-normal-*-14-*-*-*-*-*-*"
#define SANS_20	"-adobe-helvetica-bold-r-normal-*-20-*-*-*-*-*-*"
#define SANS_34	"-adobe-helvetica-bold-r-normal-*-34-*-*-*-*-*-*"
#define ROM_14  "-adobe-times-bold-r-normal-*-14-*-*-*-*-*-*"
#define ROM_20	"-adobe-times-bold-r-normal-*-20-*-*-*-*-*-*"
#define ROM_34  "-adobe-times-bold-r-normal-*-34-*-*-*-*-*-*"
#define SYS_14  "-adobe-courier-bold-r-normal-*-14-*-*-*-*-*-*"
#define SYS_20	"-adobe-courier-bold-r-normal-*-20-*-*-*-*-*-*"
#define SYS_34  "-adobe-courier-bold-r-normal-*-34-*-*-*-*-*-*"

// From the above list, pick three: one for system, one for sans, one for roman
//  These will be used by the graphics context - 14 is the default point
#define SYS_FONT_NAME SYS_14
#define SANS_FONT_NAME SANS_14
#define ROMAN_FONT_NAME ROM_14

// constants for font scaling
const int LARGE_FONT_SIZE = 30;
const int FONT_SIZE_MAX   = 100;
const int MAX_FONT_DIFF   = 30;

// Our function wrapper.  We send it argc, argv
EXPORT int chi_main(int, char**);

// Global
Window win;
Display* display;
int screen_num;
GC xgc;
PrivatePixmap ppm;

// Make these global.  Screen dimensions consistent with other chigc's.
static int DEF_WIDTH = 640, DEF_HEIGHT = 480;

#ifndef EXPORT

/* CLASSES *****************************************************************/
EXPORT class PrivatePixmap
// PURPOSE: Provides pixmap memory for screen redraws
// NOTES:  Make sure global xgc has been initialized.
{
public:
	PrivatePixmap();
	void init();
	void close();
	void repaint() const;
	void clear(unsigned long color) const;
	Pixmap id() const;
private:
	Pixmap _ppm;
};


//**************************************************************************
EXPORT class Chi_GraphicsContext
// PURPOSE: Sends graphics primitives to a display window
{
public:
   enum Color
   {  BLACK, BLUE, GREEN, CYAN,
      RED, MAGENTA, BROWN, LIGHTGRAY,  /* dark colors */
      DARKGRAY, LIGHTBLUE, LIGHTGREEN, LIGHTCYAN,
      LIGHTRED, LIGHTMAGENTA, YELLOW, WHITE,  /* light colors */
      DEFAULT_COLOR = BLACK 
         /* default color for drawing--opposite from background */
   }; 
   enum PenThickness { THIN_PEN = 0, THICK_PEN = 3 };
   enum PenStyle { SOLID_PEN, DOTTED_PEN, DASHED_PEN };
   enum BrushStyle { EMPTY_BRUSH, SOLID_BRUSH, HORIZ_BRUSH, FDIAG_BRUSH,
                     BDIAG_BRUSH, XDIAG_BRUSH };
   enum FontStyle { SYS_FONT, SANS_FONT, ROMAN_FONT }; // these should stay 0, 1, 2
   enum RopMode { COPY_ROP, XOR_ROP, OR_ROP, AND_ROP, NOT_ROP };

   Chi_GraphicsContext();
   Chi_GraphicsContext(double x1, double y1, double x2, double y2);

   void close();
   void clear_display() const;

//  Shapes
   void point(double x, double y) const;
   void line(double xfrom, double yfrom, double xto, double yto) const;
   void arc(double x, double y, double ra, double rb, double afrom,
            double ato) const;
   void rectangle(double xleft, double ytop, double xright, double ybot) const;
   void ellipse(double x, double y, double ra, double rb) const;
   void piesegment(double x, double y, double ra, double rb,
                   double afrom, double ato) const;

   void text(const Chi_String t, double x, double y);
   void text_extent(const Chi_String t, double& x, double& y) const;

   void set_brush_style(BrushStyle style);
   void set_brush_color(Color color);
   void set_pen_style(PenStyle style);
   void set_pen_thickness(PenThickness thickness);
   void set_pen_color(Color color);
   void set_font_size(double size);
   void set_font_style(FontStyle style);
   void set_rop_mode(RopMode mode);
   RopMode get_rop_mode();
   void set_window_coord(double xmin, double ymin, double xmax, double ymax);

private:
   void common_init();
   unsigned long xwin_color(Color color) const;
   int user_to_disp_x(double x) const;
   int user_to_disp_y(double y) const;
   int user_to_disp_dx(double x) const;
   int user_to_disp_dy(double y) const;
   double disp_to_user_dx(int x) const;
   double disp_to_user_dy(int y) const;

   // DDC 1-12-95 **** functions for font scaling ****
   void scaled_text(const char t[], int x, int y);  
   Pixmap scale_text(Pixmap& textpm, int tx, int ty, 
                     int newtx, int newty);
   unsigned char* PixmapToBits(Pixmap& textpm, int tx, int ty);
   void BitsToPixmap(Pixmap& newTextpm, unsigned char* newBits, 
                     int newtx, int newty);
   void scale_bits(unsigned char*, int, int, unsigned char*, int, int);
   void get_large_font();
   void GetCoordFontSize();

   double _user_xmin;            // the user's window's logical window coords
   double _user_xmax;
   double _user_ymin;
   double _user_ymax;
   int _disp_xmax;               // the physical window dimension (in pixels)
   int _disp_ymax;
   Color _brush_color;           // for filling, etc.
   BrushStyle _brush_style;      // for filling, etc
   Color _pen_color;             // color we draw text, graphics with
   PenStyle _pen_style;          // dashed, solid, etc
   PenThickness _pen_thickness;  // thick or thin
   Color _bg_color;              // background color (usually white)

   
   char _fontbuff[128];   // current font string - DDC 1/13/96
   FontStyle _font_style; // current font style  - DDC 1/13/96
   Bool   _font_loaded;   // tells if a font is loaded or not - DDC 1/15/96
   double _font_size;     // font size in point number - DDC 1/15/96
   double _font_scale;    // font scale factor 1 = 14 point. - DDC 1/17/96
   int _coord_font_size;  // font point for size 1 in current coord system - DDC 3/28/96
   int _large_font_size;  // font size of large font used for scaling - DDC 3/27/96
   int _height_pixels;    // pixel size of font about 12 point - DDC 3/28/96
   int _height_size;      // size of font in case 12 pt isn't there - DDC 3/28/96     

   RopMode _rop_mode;     // current ROP mode - DDC 1/17/96
   XFontStruct* _fontinfo_ptr; 	// points to font info for all fonts used
   Pixmap _fill_stipple;		// for various drawing fill styles
};

#endif

/* OPERATIONS **************************************************************/
PrivatePixmap::PrivatePixmap()
// PURPOSE: Default constructor.  We defer data initialization.
{}

//..........................................................................
void PrivatePixmap::init()
// PURPOSE: Allocates an X pixmap
{
	_ppm = XCreatePixmap(display, win, DEF_WIDTH, DEF_HEIGHT, DefaultDepth(display, screen_num));
}

//..........................................................................
void PrivatePixmap::close()
// PURPOSE: Returns pixmap memory to the freestore.
{
	XFreePixmap(display, _ppm);
}

//..........................................................................
void PrivatePixmap::repaint() const
// PURPOSE: Puts the image of the pixmap onto the display screen.
{
	XCopyArea(display, _ppm, win, xgc, 0, 0, DEF_WIDTH, DEF_HEIGHT, 0, 0);
}

//..........................................................................
void PrivatePixmap::clear(unsigned long color) const
// PURPOSE: Clears the pixmap, clearing with color color.
{
		  XGCValues vals;
		  XGetGCValues(display, xgc, GCForeground, &vals);

	unsigned long fg = vals.foreground;
	XSetForeground(display, xgc, color);
	XFillRectangle(display, _ppm, xgc, 0, 0, DEF_WIDTH + 1, DEF_HEIGHT + 1);
	XSetForeground(display, xgc, fg);
}

//..........................................................................
Pixmap PrivatePixmap::id() const
// PURPOSE: Returns the pixmap id.
{
	return _ppm;
}

//**************************************************************************
Chi_GraphicsContext::Chi_GraphicsContext()
// PURPOSE: Initializes user coordinates
// RECEIVES: The logical coordinates for your window
// NOTES: Initializes both GC and pixmap drawing tools
:   _user_xmin(0), _user_ymin(0), _user_xmax(20), _user_ymax(15),
	_brush_color(BLACK), _brush_style(EMPTY_BRUSH), _pen_color(BLACK),
	_pen_style(SOLID_PEN), _pen_thickness(THIN_PEN), _bg_color(WHITE),
	_disp_xmax(DEF_WIDTH), _disp_ymax(DEF_HEIGHT)
{
	common_init();
}

//..........................................................................
Chi_GraphicsContext::Chi_GraphicsContext(double x1, double y1, double x2, double y2)
// PURPOSE: Initializes user coordinates
// RECEIVES: The logical coordinates for your window
// NOTES: Initializes both GC and pixmap drawing tools
:   _user_xmin(x1), _user_ymin(y1), _user_xmax(x2), _user_ymax(y2),
  _brush_color(BLACK), _brush_style(EMPTY_BRUSH), _pen_color(BLACK),
	_pen_style(SOLID_PEN), _pen_thickness(THIN_PEN), _bg_color(WHITE),
	_disp_xmax(DEF_WIDTH), _disp_ymax(DEF_HEIGHT)
{
	common_init();
}

//...........................................................................
void Chi_GraphicsContext::common_init()
// PURPOSE:  common code for multiple constructors
{

	// specify black foreground since default window background is
	//   white and default foreground is undefined.
	XSetForeground(display, xgc, BlackPixel(display, screen_num));

	// set line attributes.  This class only uses rounded styles.
	XSetLineAttributes(display, xgc, _pen_thickness, LineSolid,
			CapRound, JoinRound);

  // set up the font - changed 1/13/96 for sizable fonts - DDC
  strcpy(_fontbuff, SANS_FONT_NAME);
  _font_style = SANS_FONT;

  _font_scale = 1;  // the scale factor of the font
  _font_loaded = FALSE;
  GetCoordFontSize();
  set_font_size(1);

  set_rop_mode(COPY_ROP);

  clear_display();
}

//...........................................................................
void Chi_GraphicsContext::close()
// PURPOSE:    Deallocates various objects when gc is through.
// NOTES:      This does NOT close down the private pixmap--only deallocates
//             the tools this gc uses.
{
   if (_brush_style != EMPTY_BRUSH && _brush_style != SOLID_BRUSH)  // don't free what isn't allocated
      XFreePixmap(display, _fill_stipple);
 
   if(_fontinfo_ptr)
      XFreeFont(display, _fontinfo_ptr);
}

//...........................................................................
void Chi_GraphicsContext::clear_display() const
// PURPOSE:    Clears the DC the gc is in, and the pixmap too.
{
	XClearWindow(display, win);
	ppm.clear(xwin_color(WHITE));
}

//...........................................................................
void Chi_GraphicsContext::set_window_coord(double xmin, double ymin,
		double xmax, double ymax)
/* PURPOSE:    Set the logical coordinate system
	RECEIVES:   xmin, ymin - coordinates of the top left corner
					xmax, ymax - coordinates of the bottom right corner
*/
{  _user_xmin = xmin;
   _user_xmax = xmax;
   _user_ymin = ymin;
   _user_ymax = ymax;
   GetCoordFontSize();
}

//...........................................................................
int Chi_GraphicsContext::user_to_disp_x(double x) const
// PURPOSE:    scales the x coordinate from user to display coordinates
// RECEIVES:   x - a coordinate position in user coordinates
// RETURNS:    the scaled coordinate
{  return (int) ((x - _user_xmin) * _disp_xmax / (_user_xmax - _user_xmin));
}

//...........................................................................
int Chi_GraphicsContext::user_to_disp_y(double y) const
// PURPOSE:    scales the y coordinate from user to display coordinates
// RECEIVES:   y - a coordinate position in user coordinates
// RETURNS:    the scaled coordinate
{  return (int) ((y - _user_ymin) * _disp_ymax / (_user_ymax - _user_ymin));
}

//...........................................................................
int Chi_GraphicsContext::user_to_disp_dx(double x) const
// PURPOSE:    scales an x distance from user to display coordinates
// RECEIVES:   x - a distance in user coordinates
// RETURNS:    the scaled distance
{  return (int) (x * _disp_xmax / (_user_xmax - _user_xmin));
}

//...........................................................................
int Chi_GraphicsContext::user_to_disp_dy(double y) const
// PURPOSE:    scales an y distance from user to display coordinates
// RECEIVES:   y - a distance in user coordinates
// RETURNS:    the scaled distance
{  return (int) (y * _disp_ymax / (_user_ymax - _user_ymin));
}

//...........................................................................
double Chi_GraphicsContext::disp_to_user_dx(int x) const
// PURPOSE:    scales an x distance from display to user coordinates
// RETURNS:    the scaled distance
{  return (double)x * (_user_xmax - _user_xmin) / _disp_xmax;
}

//...........................................................................
double Chi_GraphicsContext::disp_to_user_dy(int y) const
// PURPOSE:    scales an y distance from user to display coordinates
// RETURNS:    the scaled distance
{  return (double) y * (_user_ymax - _user_ymin) / _disp_ymax;
}

//...........................................................................
void Chi_GraphicsContext::point(double x, double y) const
/* PURPOSE:    plot a point
	RECEIVES:   x, y - the point coordinates
*/
{  const int POINT_RADIUS = 2;
	int disp_x = user_to_disp_x(x);
	int disp_y = user_to_disp_y(y);

  XSetForeground(display, xgc, xwin_color(_brush_color));
	XDrawArc(display, win, xgc, disp_x - POINT_RADIUS, disp_y - POINT_RADIUS,
		POINT_RADIUS, POINT_RADIUS, 0 , 360 * 64);
	XDrawArc(display, ppm.id(), xgc, disp_x - POINT_RADIUS,
		disp_y - POINT_RADIUS, POINT_RADIUS, POINT_RADIUS, 0 , 360 * 64);

	XSetForeground(display, xgc, xwin_color(_pen_color));
	XDrawArc(display, win, xgc, disp_x - POINT_RADIUS, disp_y - POINT_RADIUS,
		POINT_RADIUS, POINT_RADIUS, 0 , 360 * 64);
	XDrawArc(display, ppm.id(), xgc, disp_x - POINT_RADIUS,
		disp_y - POINT_RADIUS, POINT_RADIUS, POINT_RADIUS, 0 , 360 * 64);
}

//...........................................................................
void Chi_GraphicsContext::ellipse(double x, double y, double ra, double rb) const
/* PURPOSE:    Plot an ellipse
	RECEIVES:   x, y - center
					ra, rb - radii in x- and y- direction
*/
{
   int disp_x = user_to_disp_x(x);
   int disp_y = user_to_disp_y(y);
   int disp_rx = user_to_disp_dx(ra);
   int disp_ry = user_to_disp_dy(rb);

   if (disp_rx < 0) disp_rx = -disp_rx;  
   if (disp_ry < 0) disp_ry = -disp_ry;

   if (_brush_style != EMPTY_BRUSH)
   {  XSetForeground(display, xgc, xwin_color(_brush_color));
      XFillArc(display, win, xgc, disp_x - disp_rx, disp_y - disp_ry,
         disp_rx * 2, disp_ry * 2, 0, 360 * 64);
      XFillArc(display, ppm.id(), xgc, disp_x - disp_rx, disp_y - disp_ry,
         disp_rx * 2, disp_ry * 2, 0, 360 * 64);
   }

   XSetForeground(display, xgc, xwin_color(_pen_color));
   XDrawArc(display, win, xgc, disp_x - disp_rx, disp_y - disp_ry,
      disp_rx * 2, disp_ry * 2, 0, 360 * 64);
   XDrawArc(display, ppm.id(), xgc, disp_x - disp_rx, disp_y - disp_ry,
      disp_rx * 2, disp_ry * 2, 0, 360 * 64);
}

//...........................................................................
void Chi_GraphicsContext::line(double xfrom, double yfrom, double xto,
				double yto) const
/* PURPOSE:    plot a line segment with current pen style and color
	RECEIVES:   xfrom, yfrom - starting point
					xto, yto - ending point
*/
{
	XSetForeground(display, xgc, xwin_color(_pen_color));
	XDrawLine(display, win, xgc, user_to_disp_x(xfrom), user_to_disp_y(yfrom),
		user_to_disp_x(xto), user_to_disp_y(yto));
	XDrawLine(display, ppm.id(), xgc, user_to_disp_x(xfrom),
		user_to_disp_y(yfrom), user_to_disp_x(xto), user_to_disp_y(yto));
}

//...........................................................................
void Chi_GraphicsContext::arc(double x, double y, double ra, double rb,
				double afrom, double ato) const
/* PURPOSE:    Plot an arc
	RECEIVES:   x, y - center
					ra, rb - radii in x- and y- direction
					afrom, ato - starting, ending angles
*/
{
	ato -= afrom;  // correct for X's coordinate scheme
  int disp_x = user_to_disp_x(x);
	int disp_y = user_to_disp_y(y);
	int disp_rx = user_to_disp_dx(ra);
	int disp_ry = user_to_disp_dy(rb);

	XSetForeground(display, xgc, xwin_color(_pen_color));
	XDrawArc(display, win, xgc, disp_x - disp_rx, disp_y - disp_ry,
		disp_rx * 2, disp_ry * 2, (int) (afrom * 64), (int) (ato * 64));
	XDrawArc(display, ppm.id(), xgc, disp_x - disp_rx, disp_y - disp_ry,
		disp_rx * 2, disp_ry * 2, (int) (afrom * 64), (int) (ato * 64));
}

//...........................................................................
void Chi_GraphicsContext::piesegment(double x, double y, double ra,
				  double rb, double afrom, double ato) const
/* PURPOSE:   Plot a pie segment
   RECEIVES:  x, y - center
	      ra, rb - radii in x- and y- direction
	      afrom, ato - starting, ending angles
*/
{
	const double RADS = 3.14159265358979323846 / 180.0;


  double new_ato = ato - afrom;
	int disp_x = user_to_disp_x(x);
	int disp_y = user_to_disp_y(y);
	int disp_rx = user_to_disp_dx(ra);
	int disp_ry = user_to_disp_dy(rb);

	if (_brush_style != EMPTY_BRUSH)
	{	
    XSetForeground(display, xgc, xwin_color(_brush_color));
  	XFillArc(display, win, xgc, disp_x - disp_rx, disp_y - disp_ry,
			disp_rx * 2, disp_ry * 2, (int) (afrom * 64), (int) (new_ato * 64));
		XFillArc(display, ppm.id(), xgc, disp_x - disp_rx, disp_y - disp_ry,
			disp_rx * 2, disp_ry * 2, (int) (afrom * 64), (int) (new_ato * 64));
	}
  else
  {
  	XSetForeground(display, xgc, xwin_color(_pen_color));
		XDrawArc(display, win, xgc, disp_x - disp_rx, disp_y - disp_ry,
	 		disp_rx * 2, disp_ry * 2, (int) (afrom * 64), (int) (new_ato * 64));
		XDrawArc(display, ppm.id(), xgc, disp_x - disp_rx, disp_y - disp_ry,
	 		disp_rx * 2, disp_ry * 2, (int) (afrom * 64), (int) (new_ato * 64));
 
		double endpt1_x = disp_x + cos(afrom * RADS) * disp_rx;
		double endpt1_y = disp_y - sin(afrom * RADS) * disp_ry;
		double endpt2_x = disp_x + cos(ato * RADS) * disp_rx;
		double endpt2_y = disp_y - sin(ato * RADS) * disp_ry;
 
		XDrawLine(display, win, xgc, disp_x, disp_y, (int) endpt1_x,
			(int) endpt1_y);
 		XDrawLine(display, ppm.id(), xgc, disp_x, disp_y, (int) endpt1_x,
			(int) endpt1_y);
 		XDrawLine(display, win, xgc, disp_x, disp_y, (int) endpt2_x,
			(int) endpt2_y);
		XDrawLine(display, ppm.id(), xgc, disp_x, disp_y, (int) endpt2_x,
 			(int) endpt2_y);
	}
}

//...........................................................................
void Chi_GraphicsContext::rectangle(double xleft, double ytop,
				double xright, double ybottom) const
/* PURPOSE:    Plot a rectangle
   RECEIVES:   xleft, ytop - the top left corner
               xright, ybottom - the bottom right corner
*/
{
	int disp_xleft = user_to_disp_x(xleft);
	int disp_ytop = user_to_disp_y(ytop);
	int disp_xright = user_to_disp_x(xright);
	int disp_ybottom = user_to_disp_y(ybottom);

	if (disp_xleft > disp_xright)
	{  int temp = disp_xleft; disp_xleft = disp_xright; disp_xright = temp;
	}

	if (disp_ytop > disp_ybottom)
	{  int temp = disp_ytop; disp_ytop = disp_ybottom; disp_ybottom = temp; 
	}

	if (_brush_style != EMPTY_BRUSH)
	{  XSetForeground(display, xgc, xwin_color(_brush_color));
		XFillRectangle(display, win, xgc, disp_xleft, disp_ytop,
			disp_xright - disp_xleft, disp_ybottom - disp_ytop);
		XFillRectangle(display, ppm.id(), xgc, disp_xleft, disp_ytop,
			disp_xright - disp_xleft, disp_ybottom - disp_ytop);
	}

	XSetForeground(display, xgc, xwin_color(_pen_color));
	XDrawRectangle(display, win, xgc, disp_xleft, disp_ytop,
		disp_xright - disp_xleft, disp_ybottom - disp_ytop);
	XDrawRectangle(display, ppm.id(), xgc, disp_xleft, disp_ytop,
		disp_xright - disp_xleft, disp_ybottom - disp_ytop);
}

//...........................................................................
void Chi_GraphicsContext::set_brush_style(BrushStyle style)
/*    PURPOSE:    sets the style for the brush to color the insides of
					closed shapes (rectangles, polygons, ellipses)
      RECEIVES:   style - the brush style
      NOTES:      No pixmap allocation error checking.  Round up the usual suspects in case of misbehavior.
*/
{
static unsigned char bdiag_bits[] = {
   0x01, 0x00, 0x00, 0x03, 0x00, 0x00, 0x07, 0x00, 0x00, 0x0e, 0x00, 0x00,
   0x1c, 0x00, 0x00, 0x38, 0x00, 0x00, 0x70, 0x00, 0x00, 0xe0, 0x00, 0x00,
   0xc0, 0x01, 0x00, 0x80, 0x03, 0x00, 0x00, 0x07, 0x00, 0x00, 0x0e, 0x00,
   0x00, 0x1c, 0x00, 0x00, 0x38, 0x00, 0x00, 0x70, 0x00, 0x00, 0xe0, 0x00,
   0x00, 0xc0, 0x01, 0x00, 0x80, 0x03, 0x00, 0x00, 0x07, 0x00, 0x00, 0x0e};


static unsigned char fdiag_bits[] = {
   0x00, 0x00, 0x0e, 0x00, 0x00, 0x0e, 0x00, 0x80, 0x0f, 0x00, 0xc0, 0x07,
   0x00, 0xe0, 0x03, 0x00, 0xf8, 0x00, 0x00, 0x7c, 0x00, 0x00, 0x3e, 0x00,
   0x80, 0x0f, 0x00, 0xc0, 0x0f, 0x00, 0xc0, 0x07, 0x00, 0xe0, 0x03, 0x00,
   0xe0, 0x01, 0x00, 0xf0, 0x00, 0x00, 0x78, 0x00, 0x00, 0x38, 0x00, 0x00,
   0x3c, 0x00, 0x00, 0x1f, 0x00, 0x00, 0x0f, 0x00, 0x00, 0x07, 0x00, 0x00};

static unsigned char horz_bits[] = {
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0xff, 0xff, 0x0f, 0xff, 0xff, 0x0f, 0xff, 0xff, 0x0f, 0xff, 0xff, 0x0f,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
	
static unsigned char xdiag_bits[] = {
   0x07, 0x00, 0x0e, 0x1f, 0x00, 0x0f, 0x1f, 0x80, 0x07, 0x3c, 0xc0, 0x03,
   0x78, 0xc0, 0x01, 0xf0, 0xe0, 0x00, 0xe0, 0xe0, 0x00, 0xc0, 0x70, 0x00,
   0x80, 0x1b, 0x00, 0x00, 0x1f, 0x00, 0x00, 0x1f, 0x00, 0xc0, 0x3f, 0x00,
   0xc0, 0x3f, 0x00, 0xe0, 0x70, 0x00, 0x78, 0x70, 0x00, 0x3c, 0xc0, 0x01,
   0x0e, 0xc0, 0x01, 0x0f, 0x80, 0x07, 0x07, 0x00, 0x0f, 0x03, 0x00, 0x0e};

//   static char vert_bits[] =  { 0x04, 0x04, 0x04, 0x04, 0x04 };  //cbm do we have vertical lines???
   static int dim = 20; // make sure this changes if bitmap files change!!!
   static int sdepth = 1; // a stipple has depth 1
   
   
   if (_brush_style != EMPTY_BRUSH && _brush_style != SOLID_BRUSH)  // don't free what isn't allocated
      XFreePixmap(display, _fill_stipple);

   _brush_style = style;

 // cbm Why didn't I simply place the choices into an N x 5 array, then assign bitchoice = all_bits[k], where
 //   k is the choice?  Because Hollow and Solid brushes don't even deal with stipples.  The cascading "if"
 //   statements seemed more reasonable than storing nonsense data into an "all_bits" array (which would look
 //   a little less kludgey).


   if ((_brush_style == EMPTY_BRUSH) || (_brush_style == SOLID_BRUSH))
      XSetFillStyle(display, xgc, FillSolid);
   else
   {  char* bitchoice;
      if (style == HORIZ_BRUSH) 
		     bitchoice = (char*) horz_bits;
      else if (style == FDIAG_BRUSH) 
		     bitchoice = (char*) fdiag_bits;
      else if (style == BDIAG_BRUSH) 
		     bitchoice = (char*) bdiag_bits;
      else if (style == XDIAG_BRUSH) 
		     bitchoice = (char*) xdiag_bits;
      else 
		     cerr << "Illegal brush coice";  // we chasitise, then keep right on going.  Beware!

			XGCValues v;
		  XGetGCValues(display, xgc, GCForeground | GCBackground, &v);

      _fill_stipple = XCreatePixmapFromBitmapData(display, win, bitchoice, dim,
		     dim, v.foreground, v.background, sdepth);
      XSetStipple(display, xgc, _fill_stipple);
      XSetFillStyle(display, xgc, FillStippled);
   }
}

//...........................................................................
void Chi_GraphicsContext::set_brush_color(Color color)
/* PURPOSE:    set the color of the brush that colors the insides of
					closed shapes (rectangles, polygons, ellipses)
	RECEIVES:   color - the brush color
	NOTES:      only changes the class's variable: color is changed in xgc
					at drawing time.
*/
{
	_brush_color = color;
}

//...........................................................................
void Chi_GraphicsContext::set_pen_style(PenStyle style)
/* PURPOSE:    set the style (dotted, dashed, solid) of the pen that draws
					shapes (lines, points, polygons)
	RECEIVES:   style - the pen style
*/
{
	static char dash_list[] = { 4, 4 };    // line definitions
	static int dash_list_len = 2;
	static char dot_list[] = { 3, 1 };
	static int dot_list_len = 2;

	int gc2xwin[]= { LineSolid, LineOnOffDash, LineOnOffDash };
					// SOLID_PEN, DOTTED_PEN, DASHED_PEN

	_pen_style = style;

	if (_pen_style == DOTTED_PEN)
		XSetDashes(display, xgc, 0, dot_list, dot_list_len);
	else if (_pen_style == DASHED_PEN)
		XSetDashes(display, xgc, 0, dash_list, dash_list_len);
	
	XSetLineAttributes(display, xgc, _pen_thickness, gc2xwin[_pen_style],
		CapRound, JoinRound);
}

//...........................................................................
void Chi_GraphicsContext::set_pen_thickness(PenThickness thickness)
/* PURPOSE:    set the thickness (thick or thin) of the pen that draws
					shapes (lines, points, polygons)
	RECEIVES:   thickness - the pen thickness
*/
{
	_pen_thickness = thickness;
	
	XGCValues gc_values;
	XGetGCValues(display, xgc, GCLineStyle, &gc_values);
	XSetLineAttributes(display, xgc, _pen_thickness, gc_values.line_style,
		CapRound, JoinRound);
}

//...........................................................................
void Chi_GraphicsContext::set_pen_color(Color color)
/* PURPOSE:    set the color of the pen that draws all shapes
	RECEIVES:   color- the pen color
*/
{
	_pen_color = color;
}

//...........................................................................
void Chi_GraphicsContext::set_font_size(double s)
/* PURPOSE:    set the font size for text output
   RECEIVES:   s - the font size (1.0 = normal)
*/
{
   _font_scale = s;
   Bool done = FALSE;
   int inc = 0;
   int size = int(_coord_font_size * s);
   _font_size = size;
   _font_loaded = FALSE;
   XFontStruct* lowerFontPtr = 0;
   int upperSize = 0, lowerSize = 0;

   size = int(_font_size);

   inc = 0;

   while (!done)
   {
      switch (_font_style)
      {
         case ROMAN_FONT:
            sprintf(_fontbuff, "-adobe-times-bold-r-normal-*-%d-*-*-*-*-*-*", size);
            break;
         case SANS_FONT:
            sprintf(_fontbuff, "-adobe-helvetica-bold-r-normal-*-%d-*-*-*-*-*-*", size);
            break;
         case SYS_FONT:
            sprintf(_fontbuff, "-adobe-courier-bold-r-normal-*-%d-*-*-*-*-*-*", size);
            break;
      }
      _fontinfo_ptr = XLoadQueryFont(display, _fontbuff); // load large font
   
      if ((_fontinfo_ptr == NULL) && (inc < MAX_FONT_DIFF))
      {
         size++; 
         inc++;
      }
      else // we got a high font, we're done!
      {  
         done = TRUE; 
         upperSize = size;
      }
   }

   done = FALSE;
   inc = 0;
   size = int(_font_size);
   while (!done)
   {
      switch (_font_style)
      {
         case ROMAN_FONT:
            sprintf(_fontbuff, "-adobe-times-bold-r-normal-*-%d-*-*-*-*-*-*", size);
            break;
         case SANS_FONT:
            sprintf(_fontbuff, "-adobe-helvetica-bold-r-normal-*-%d-*-*-*-*-*-*", size);
            break;
         case SYS_FONT:
            sprintf(_fontbuff, "-adobe-courier-bold-r-normal-*-%d-*-*-*-*-*-*", size);
            break;
      }
      lowerFontPtr = XLoadQueryFont(display, _fontbuff); // use dummy font pointer
   
      if ((lowerFontPtr == NULL) && (size > 1))
      {
         size--;
         inc++;   // not currently used in this loop 
      }
      else // we got a low font, we're done!
      {  
         done = TRUE; 
         lowerSize = size;
      }
   }

   // select font closer to the requested size  
   if ((!_fontinfo_ptr) || ((upperSize - _font_size) > (_font_size - lowerSize)) && lowerFontPtr)
      _fontinfo_ptr = XLoadQueryFont(display, _fontbuff);  // load smaller font 



   if (_fontinfo_ptr)
   {
       XSetFont(display, xgc, _fontinfo_ptr->fid);
      _font_loaded = TRUE;
   }
   else
   {
      cerr << "Problems: Can't load any fonts!" << endl << "reqested size: " << _font_size
           << endl << "upper size limit checked: " << upperSize 
           << endl << "lower size limit checked: " << lowerSize << endl;
      exit(1);
   }

   if(lowerFontPtr)
      XFreeFont(display, lowerFontPtr);

   return;
}

//...........................................................................
void Chi_GraphicsContext::set_font_style(FontStyle style)
/* PURPOSE:    set the font style for text output
	RECEIVES:   style - the style with which to write text.
	REMARKS:    If the style requested is SYSTEM_FONT, font size is reset
					to "normal."
*/
{  XFreeFont(display, _fontinfo_ptr);
	 
   _font_style = style; // 1/13/96 - DDC, for sizable fonts

   char font[120];
   if (style == SYS_FONT)
      strcpy(font, SYS_FONT_NAME);
   else if (style == SANS_FONT)
      strcpy(font, SANS_FONT_NAME);
   else
      strcpy(font, ROMAN_FONT_NAME);

   _fontinfo_ptr = XLoadQueryFont(display, font);
	 if (_fontinfo_ptr == NULL)
   { cerr << "Unable to initialize font: " << font << endl;
     exit(1);
	 }
   XSetFont(display, xgc, _fontinfo_ptr->fid);
}

//...........................................................................
void Chi_GraphicsContext::text(const Chi_String t, double x, double y)
/* PURPOSE:    plot text in the current font
   RECEIVES:   t - the text
	       x, y - coordinates to start plotting
*/
{
   int disp_x = user_to_disp_x(x);
   int disp_y = user_to_disp_y(y);

   if (_font_loaded) // if we have a font
   {
      XSetForeground(display, xgc, xwin_color(_pen_color));
      XDrawString(display, win, xgc, disp_x, disp_y, t.c_str(), t.length());
      XDrawString(display, ppm.id(), xgc, disp_x, disp_y, t.c_str(), t.length());
   }
   else
      scaled_text(t.c_str(), disp_x, disp_y);
}

//...........................................................................
void Chi_GraphicsContext::text_extent(const Chi_String t, double& x,	
                                      double& y) const
/* PURPOSE:    measure text in the current font
   RECEIVES:   t - the text
	       x, y (OUT) - the size in x- and y-direction
*/
{
   int direction, ascent, descent;
   XCharStruct overall;

   XTextExtents(_fontinfo_ptr, t.c_str(), t.length(), &direction, &ascent, &descent,
                &overall);
   x = disp_to_user_dx(overall.width);
   y = disp_to_user_dy(ascent + descent); 
}

//...........................................................................
void Chi_GraphicsContext::set_rop_mode(RopMode rop)
/* PURPOSE:    set the mode for raster operations
	RECEIVES:   rop - the new rop mode
*/
{
   _rop_mode = rop;
   int gc2xwin[] =
   {  GXcopy , GXxor, GXor, GXand,  GXcopyInverted  };            //GXcopyInverted };
   // COPY_ROP, XOR_ROP, OR_ROP, AND_ROP, NOT_ROP
   XSetFunction(display, xgc, gc2xwin[rop]);
}
//...........................................................................
Chi_GraphicsContext::RopMode Chi_GraphicsContext::get_rop_mode()
/* PURPOSE: get the mode for raster operations
   RETURNS: the current ROP mode  
*/
{  return _rop_mode;
}
//...........................................................................
unsigned long Chi_GraphicsContext::xwin_color(Color col) const
// PURPOSE:    Returns the pixel value corresponding to color col.
// RECEIVED:   A color to translate.  We use the R3 database.
{
	static char* r3db[16] = {
		"black", "blue", "green", "dark turquoise",
		"firebrick", "purple", "brown", "light gray",
		"dark slate gray", "light steel blue", "lime green", "cyan",
		"red", "magenta", "yellow", "white"
	};

	XColor cell_def, rgb_def;
	Colormap cmap = DefaultColormap(display, screen_num);

	if (!XAllocNamedColor(display, cmap, r3db[col], &cell_def, &rgb_def))
	{  cerr << "Unable to allocate color" << endl;
		exit(1);
	}
	return cell_def.pixel;
}

/***************************************************************************/

unsigned char* Chi_GraphicsContext::PixmapToBits(Pixmap& textpm, 
                                              int tx, int ty)
//PURPOSE:  To convert the pixmap to bits that can be scaled
//RECEIVES: pixmap of text, and integer dimentions tx,ty
//RETURNS:  The bits - an unsigned character array
{
   unsigned char* cptr;
   unsigned long pixel;

   cptr    = new unsigned char[tx*ty];
   XImage* ximage;

   ximage = XGetImage(display, textpm, 0, 0, tx, ty, 
                      1, /* plane_mask for pixmap depth */
                      ZPixmap); 
   if (!ximage)
   {
      cerr << "ERROR: failed reading XImage" << endl;
      exit(1);
   }

   for(int i=0; i<(tx*ty); i++)
   {
      cptr[i] = ximage->data[i];
   }

   XDestroyImage(ximage);
   return cptr;
}

/***************************************************************************/

void Chi_GraphicsContext::BitsToPixmap(Pixmap& newTextpm, unsigned char* newBits, 
                                   int newtx, int newty)
//PURPOSE: To convert bits back to a pixmap
//RECEIVES: the (new) pixmap, the (new) bits to be converted, int dimentions
{

   XImage* ximage;
   ximage = XGetImage(display, newTextpm, 0, 0, newtx, newty, 
                      1, /* plane_mask for pixmap depth */
                      ZPixmap); 

/* modify new XImage data field with new scaled bits */
   ximage->data = (char*)newBits;

/* put image on new pixmap and destroy XImage */
   XPutImage(display, newTextpm, xgc, ximage, 0, 0, 0, 0, newtx, newty);
   XDestroyImage(ximage);
}

/***************************************************************************/

void Chi_GraphicsContext::scale_bits(unsigned char* oldBits, int h1, int v1, 
                                     unsigned char* newBits, int h2, int v2)
/* PURPOSE:    Scale a bitmap
** RECEIVES:   oldBits - location of old bitmap
**             newBits - location of new bitmap
**             h1, v1 - old horizontal and vertical size in bytes
**             h2, v2 - new horizontal and vertical size in bytes
** RETURNS:    nothing
** REMARKS:    oldBits, newBits are arrays of BITS
*/
{	
   int i1,i2,j1,j2,i1min,i1max,j1min,j1max,area,ifac,jfac;
	
   if(h1==h2 && v1==v2)
   {  memcpy(newBits, oldBits, h1 * v1);
      return;
   }
   memset(newBits,0, h2 * v2);
   for(i2=0;i2<v2;i2++)
   {	
      i1min=(i2*v1)/v2; i1max=((i2+1)*v1-1)/v2;
      for(j2=0;j2<h2;j2++)
      {  j1min=(j2*h1)/h2; j1max=((j2+1)*h1-1)/h2;
         area=0;
         for(i1=i1min;i1<=i1max;i1++)
         {  ifac=(i1==i1min ? ((i1+1)*v2-i2*v1) : i1==i1max ? ((i2+1)*v1-i1*v2) : v2);
	    if(ifac>v1) ifac=v1;
	    for(j1=j1min;j1<=j1max;j1++)
               if(oldBits[i1*h1+j1])
	       {  jfac=(j1==j1min ? ((j1+1)*h2-j2*h1) : j1==j1max ? ((j2+1)*h1-j1*h2) : h2);
	          if(jfac>h1) jfac=h1;
                  area+=ifac*jfac;
	       }
	 }
         if(2*area>=h1*v1) newBits[i2*h2+j2]=1;
      }
   }
}

/***************************************************************************/

Pixmap Chi_GraphicsContext::scale_text(Pixmap& textpm, int tx, int ty, 
                                   int newtx, int newty)
//PURPOSE:  Creates the new pixmap and scales text on it.
//RECEIVES: pixmap to scale and its dimentions. and dimentions of new pixmap
//RETURNS:  the new pixmap
{
   unsigned char* newBits = new unsigned char[newtx*newty];

   Pixmap newTextpm = XCreatePixmap(display, win, newtx, newty, 
                                    DefaultDepth(display, screen_num));

   unsigned char* bits = PixmapToBits(textpm, tx, ty);
   scale_bits (bits, tx, ty, newBits, newtx, newty);

/* clear new pixmap with white rectangle */
   XGCValues vals;
   XGetGCValues(display, xgc, GCForeground, &vals);
   unsigned long fg = vals.foreground;
   XSetForeground(display, xgc, xwin_color(Chi_GraphicsContext::WHITE));

   XFillRectangle(display, newTextpm, xgc, 0, 0, newtx, newty);
   XSetForeground(display, xgc, fg);

/* convert scaled bits back to a pixmap */
   BitsToPixmap(newTextpm, newBits, newtx, newty);

/* delete the allocated memory for bits and return the new pixmap */
   delete [] bits;  
   // NOTE: we don't delete newBits because it was deleted with the XImage
   return newTextpm;
}

/*****************************************************************************/

void Chi_GraphicsContext::get_large_font()
/*
** PURPOSE: Gets a large font in the current style and sets the _fontinfo_ptr. 
**          Used for scaling text. 
** REMARKS: This function only sets the _fontinfo_ptr, it doesn't touch
**          the user specified variables like _font_scale, etc.
*/
{
   int inc = 1;
   int size = LARGE_FONT_SIZE;
   Bool done = FALSE;  

   while (!done)
   {
      switch (_font_style)
      {
         case ROMAN_FONT:
            sprintf(_fontbuff, "-adobe-times-bold-r-normal-*-%d-*-*-*-*-*-*", size);
            break;
         case SANS_FONT:
            sprintf(_fontbuff, "-adobe-helvetica-bold-r-normal-*-%d-*-*-*-*-*-*", size);
            break;
         case SYS_FONT:
            sprintf(_fontbuff, "-adobe-courier-bold-r-normal-*-%d-*-*-*-*-*-*", size);
            break;
      }

      _fontinfo_ptr = XLoadQueryFont(display, _fontbuff);
   
      if (_fontinfo_ptr == NULL)
      {
         size += inc; // increment font
         if (inc > FONT_SIZE_MAX)
            inc = -1;

         if (size <= 1) // went up to largest and back down.
         {              // If we get here then there are no fonts!
            cerr << endl << "ERROR: No fonts available!!" << endl;
            break;
         }
      }
      else // we got a font, we're done!
      {  
         XSetFont(display, xgc, _fontinfo_ptr->fid);
         done = TRUE;
         _large_font_size = size;
      }
   }
}

/***************************************************************************/

void Chi_GraphicsContext::scaled_text(const char t[], int x, int y)
/*
** PURPOSE:  To display text from an off-screen bitmap after scaling it. 
** RECEIVED: t - the character string to scale and display
**           x,y - display coordinates for the top-left corner
*/
{
   int lf_descent;
   double large_font = 2;
   double tx, ty;    // text extent variables
   int newtx, newty; // new text extent (new pixmap width/height) vars
   Pixmap textpm, newTextpm;
   
/* Get large font for scaling */
   get_large_font();

/* get extent of font */

//   XTextExtents(_fontinfo_ptr, t.c_str(), t.length(), &direction, &ascent, &descent,
//                &overall);

   text_extent(t, tx, ty);
   lf_descent = _fontinfo_ptr->descent;

/*******************************************************************************/

newtx = int(_font_size/_large_font_size * tx);
newty = int(_font_size/_large_font_size * ty);

/**** get new dimentions for scaled font */
 /*
  if (_font_scale > 0.5)
   {
      newtx = (int)(tx * (_font_scale / 2.4));
      newty = (int)(ty * (_font_scale / 2.4));
   }
   else
   {
      newtx = (int)(tx * (_font_scale / 2)); //1.5));
      newty = (int)(ty * (_font_scale / 2)); //1.5));
   }
*/

/* make newtx and newty a multiple of 8 */
   newtx = 8 * (((newtx - 1) / 8 ) + 1);

/* the helvetica font likes the original tx to be a multiple of 3 */
//   if (_font_style == SANS_FONT)
//      tx =  3 * (((tx - 1) / 3 ) + 1);

/* the offset to allow for the font descent */
//   offset = int(ty / 5);

/* Create and clear pixmap */
   textpm = XCreatePixmap(display, win, (int)tx, (int)ty, 
                          DefaultDepth(display, screen_num));

/* clear pixmap with white rectangle */
   XGCValues vals;
   XGetGCValues(display, xgc, GCForeground, &vals);
   unsigned long fg = vals.foreground;
   XSetForeground(display, xgc, xwin_color(WHITE));
   XFillRectangle(display, textpm, xgc, 0, 0, (int)tx, (int)ty);
   XSetForeground(display, xgc, fg);
      
/* Draw string on newly created pixmap - NOTE this is set up for a large font! */
   XDrawString(display, textpm, xgc, 0, int(ty) - lf_descent, t, strlen(t));

/* new scaled pixmap */
   newTextpm = scale_text(textpm, (int)tx, (int)ty, newtx, newty);
   XCopyArea(display, textpm, newTextpm, xgc, 0, 0, (int)newtx, (int)newty, x, y);

/* copy new pixmap onto screen and PrivatePixmap */
   RopMode rop = get_rop_mode();
   set_rop_mode(XOR_ROP);
   XCopyArea(display, newTextpm, win, xgc, 0, 0, newtx, newty, x, y);
   XCopyArea(display, newTextpm, ppm.id(), xgc, 0, 0, newtx, newty, x, y);
   set_rop_mode(rop);  // restore old rop mode

/* free pixmaps we created */
   XFreePixmap(display, textpm);
   XFreePixmap(display, newTextpm);
}

/***************************************************************************/

void Chi_GraphicsContext::GetCoordFontSize()
{
   int inc = 1;
   int size = 12;
   double tx,ty;
   Bool done = FALSE;  

   while (!done)
   {
      switch (_font_style)
      {
         case ROMAN_FONT:
            sprintf(_fontbuff, "-adobe-times-bold-r-normal-*-%d-*-*-*-*-*-*", size);
            break;
         case SANS_FONT:
            sprintf(_fontbuff, "-adobe-helvetica-bold-r-normal-*-%d-*-*-*-*-*-*", size);
            break;
         case SYS_FONT:
            sprintf(_fontbuff, "-adobe-courier-bold-r-normal-*-%d-*-*-*-*-*-*", size);
            break;
      }

      _fontinfo_ptr = XLoadQueryFont(display, _fontbuff);
   
      if (_fontinfo_ptr == NULL)
      {
         size += inc; // increment font
         if (inc > FONT_SIZE_MAX)
            inc = -1;

         if (size <= 1) // went up to largest and back down.
         {              // If we get here then there are no fonts!
            cerr << endl << "ERROR: No fonts available!!" << endl;
            break;
         }
      }
      else // we got a font, we're done!
      {  
         XSetFont(display, xgc, _fontinfo_ptr->fid);
         done = TRUE;
      }
   }

   text_extent("X", tx, ty);
   _height_pixels = int(ty);
   _height_size   = size;
   _coord_font_size = int( (size)*(640/ty)*(_font_scale/(_user_ymax - _user_ymin)) );

}

/***************************************************************************/

#define BITMAPDEPTH 1
#define TOO_SMALL 0
#define BIG_ENOUGH 1

//***************************************************************************
int main(int argc, char* argv[])
{
 // First, X overhead...
	unsigned int width, height;   // window size
	unsigned int border_width = 4;

	char* window_name = "ChiLib Graphics Window";
	char* icon_name = "CGW";

	Pixmap icon_pixmap;
	XSizeHints* size_hints;
	XWMHints* wm_hints;
	XClassHint* class_hints;
	XTextProperty windowName;
	XTextProperty iconName;
	XEvent report;
	int window_size_ok = 1;
	char* display_name = 0;
	char* progname = argv[0];

 // check allocation of hints
	if (  !(size_hints = XAllocSizeHints())
		|| !(wm_hints = XAllocWMHints())
		|| !(class_hints = XAllocClassHint()))
	{
		cerr << progname << " error: failure allocating memory" << endl;
		exit(0);
	}

	progname = argv[0];

	// connect to X server
	display = XOpenDisplay(display_name);
	if (display == NULL)
	{
		cerr << progname << " error: can't connect to server " <<
			XDisplayName(display_name) << endl;
		exit(-1);
	}

	// get screen size from display structure macro
	screen_num = DefaultScreen(display);

	win = XCreateSimpleWindow(display, RootWindow(display, screen_num),
			0, 0, DEF_WIDTH, DEF_HEIGHT, border_width, BlackPixel(display,
			screen_num), WhitePixel(display, screen_num));

	/* Create pixmap of depth 1 (bitmap) for icon */
	icon_pixmap = XCreateBitmapFromData(display, win, (char*) icon_bitmap_bits,
			40, 40);

	if (XStringListToTextProperty(&window_name, 1, &windowName) == 0)
	{
		cerr << progname
				<< " error: structure allocation for windowName failed."
				<< endl;
		exit(-1);
	}

	if (XStringListToTextProperty(&icon_name, 1, &iconName) == 0) {
		cerr << progname
				<< " error: structure allocation for iconName failed."
				<< endl;
		exit(-1);
	}

	size_hints->flags = PPosition | PSize | PMinSize;
	size_hints->min_width = DEF_WIDTH;
	size_hints->min_height = DEF_HEIGHT;

	wm_hints->initial_state = NormalState;
	wm_hints->input = True;
	wm_hints->icon_pixmap = icon_pixmap;
	wm_hints->flags = StateHint | IconPixmapHint | InputHint;

	class_hints->res_name = progname;
	class_hints->res_class = "chigcx";

	XSetWMProperties(display, win, &windowName, &iconName, argv, argc,
		size_hints, wm_hints, class_hints);

	// Select desired event types
	XSelectInput(display, win, ExposureMask | KeyPressMask
		| ButtonPressMask | StructureNotifyMask);

	XGCValues xgc_values;  // ignored
	xgc = XCreateGC(display, win, 0, &xgc_values);

	XMapWindow(display, win);

  ppm.init();

	// Event loop
	static int draw_flag = TRUE;
	while (1)
	{
		XNextEvent(display, &report);
		switch (report.type)
		{
			case Expose:
				if (report.xexpose.count == 0)
				{
					if (window_size_ok)
					{  if (draw_flag == TRUE)
						{
							chi_main(argc, argv);
							draw_flag = FALSE;
						}
						else
							ppm.repaint();
					}
				}
				break;

			case ConfigureNotify:
				/* window has been resized, change width and
				 * height to send to draw_text and draw_graphics
				 * in next Expose */
				width = report.xconfigure.width;
				height = report.xconfigure.height;
				window_size_ok = (width >= size_hints->min_width)
										&& (height >= size_hints->min_height);
				break;

			case ButtonPress:
			case KeyPress:
                        /* commented out - 1/13/96 - DDC
				ppm.close();
				XFreeGC(display, xgc);
				XCloseDisplay(display);
				return 0;
                        */
			default:
				break;
		}
	}
}


#ifdef FONT_TEST1
int chi_main(int argc, char** argv)
{  Chi_GraphicsContext gc;
   char** arg_array = argv; // not used, but keeps the compiler happy.
   int arg_ct = argc;

   int k = 1;
   gc.set_window_coord(0,0,50,40);
   gc.set_brush_style(Chi_GraphicsContext::SOLID_BRUSH);
   gc.set_brush_color(Chi_GraphicsContext::GREEN);
   gc.rectangle(0,0,50,40);
   gc.set_font_style(Chi_GraphicsContext::ROMAN_FONT);
   for (double i=1; i<=2.5; i+=0.1)
   {
      gc.set_font_size(i);
      gc.text("Font sizing test", 5, k); 
      k+=(int)(2*i);
   }
   gc.close(); // always close before returning
}
#endif // FONT_TEST1

#ifdef FONT_TEST2
int chi_main(int argc, char** argv)
{  Chi_GraphicsContext gc;
   char** arg_array = argv; // not used, but keeps the compiler happy.
   int arg_ct = argc;

/*
** Tests font scaling in a weird coordinate system.
*/
   gc.set_window_coord(-40, 30, 40, -30);
   gc.set_font_style(Chi_GraphicsContext::ROMAN_FONT);
   gc.set_font_size(1.6);
   gc.rectangle(-35,10,30, -10);
   gc.text("Testing a weird coord. system: -40,30,40,-30", -30, 0);

   gc.close(); // always close before returning
 
}
#endif // FONT_TEST2
