#ifndef CHIGCX11_H
#define CHIGCX11_H

/****************************************************************************
** COPYRIGHT (C):    1994 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          Mastering Object-Oriented Design
** FILE:             chigcx.h
** PURPOSE:          Graphics context class (X-Window's version)
** VERSION           1.0
** COMPILER:         Sun CC
** TARGET:           Sun SparcStation
** PROGRAMMER:       Cay Horstmann (CSH)
**		     Charles Miller (CBM)
**                   Darrel D. Cherry (DDC)
** START DATE:       6-5-94   (CBM)
** NOTES:
	  Because X does not have a standard font system (such as those of
	  MS-Windows and Borland's BGI), the programmer must decide what
	  font to use for "system," "sans," and "roman" selections.  Also,
	  because scaling is not provided on all systems, this version of the
	  Graphics Context class does not allow for a changing of the font
	  sizes; here it is a stub.
** UPDATE HISTORY:  1-13-96 (DDC) 
          Font scaling has been implemented. There are new variables 
          and member functions in the Chi_GraphicsContext class for font
          scaling, and they are labeled as such.

****************************************************************************/

#line 37 "chigcx11.cpp"

#ifndef CHISETUP_H
#include "chisetup.h"
#endif

#line 40 "chigcx11.cpp"

#ifndef CHISTRNG_H
#include "chistrng.h"
#endif

#line 44 "chigcx11.cpp"



#include <X11/Xlib.h>


#include <X11/Xutil.h>


#include <X11/Xos.h>


#include <X11/Xatom.h>


#line 108 "chigcx11.cpp"
extern int chi_main(int, char**);

#line 120 "chigcx11.cpp"


/* CLASSES *****************************************************************/
 class PrivatePixmap
// PURPOSE: Provides pixmap memory for screen redraws
// NOTES:  Make sure global xgc has been initialized.
{
public:
	PrivatePixmap();
	void init();
	void close();
	void repaint() const;
	void clear(unsigned long color) const;
	Pixmap id() const;
private:
	Pixmap _ppm;
};


//**************************************************************************
 class Chi_GraphicsContext
// PURPOSE: Sends graphics primitives to a display window
{
public:
   enum Color
   {  BLACK, BLUE, GREEN, CYAN,
      RED, MAGENTA, BROWN, LIGHTGRAY,  /* dark colors */
      DARKGRAY, LIGHTBLUE, LIGHTGREEN, LIGHTCYAN,
      LIGHTRED, LIGHTMAGENTA, YELLOW, WHITE,  /* light colors */
      DEFAULT_COLOR = BLACK 
         /* default color for drawing--opposite from background */
   }; 
   enum PenThickness { THIN_PEN = 0, THICK_PEN = 3 };
   enum PenStyle { SOLID_PEN, DOTTED_PEN, DASHED_PEN };
   enum BrushStyle { EMPTY_BRUSH, SOLID_BRUSH, HORIZ_BRUSH, FDIAG_BRUSH,
                     BDIAG_BRUSH, XDIAG_BRUSH };
   enum FontStyle { SYS_FONT, SANS_FONT, ROMAN_FONT }; // these should stay 0, 1, 2
   enum RopMode { COPY_ROP, XOR_ROP, OR_ROP, AND_ROP, NOT_ROP };

   Chi_GraphicsContext();
   Chi_GraphicsContext(double x1, double y1, double x2, double y2);

   void close();
   void clear_display() const;

//  Shapes
   void point(double x, double y) const;
   void line(double xfrom, double yfrom, double xto, double yto) const;
   void arc(double x, double y, double ra, double rb, double afrom,
            double ato) const;
   void rectangle(double xleft, double ytop, double xright, double ybot) const;
   void ellipse(double x, double y, double ra, double rb) const;
   void piesegment(double x, double y, double ra, double rb,
                   double afrom, double ato) const;

   void text(const Chi_String t, double x, double y);
   void text_extent(const Chi_String t, double& x, double& y) const;

   void set_brush_style(BrushStyle style);
   void set_brush_color(Color color);
   void set_pen_style(PenStyle style);
   void set_pen_thickness(PenThickness thickness);
   void set_pen_color(Color color);
   void set_font_size(double size);
   void set_font_style(FontStyle style);
   void set_rop_mode(RopMode mode);
   RopMode get_rop_mode();
   void set_window_coord(double xmin, double ymin, double xmax, double ymax);

private:
   void common_init();
   unsigned long xwin_color(Color color) const;
   int user_to_disp_x(double x) const;
   int user_to_disp_y(double y) const;
   int user_to_disp_dx(double x) const;
   int user_to_disp_dy(double y) const;
   double disp_to_user_dx(int x) const;
   double disp_to_user_dy(int y) const;

   // DDC 1-12-95 **** functions for font scaling ****
   void scaled_text(const char t[], int x, int y);  
   Pixmap scale_text(Pixmap& textpm, int tx, int ty, 
                     int newtx, int newty);
   unsigned char* PixmapToBits(Pixmap& textpm, int tx, int ty);
   void BitsToPixmap(Pixmap& newTextpm, unsigned char* newBits, 
                     int newtx, int newty);
   void scale_bits(unsigned char*, int, int, unsigned char*, int, int);
   void get_large_font();
   void GetCoordFontSize();

   double _user_xmin;            // the user's window's logical window coords
   double _user_xmax;
   double _user_ymin;
   double _user_ymax;
   int _disp_xmax;               // the physical window dimension (in pixels)
   int _disp_ymax;
   Color _brush_color;           // for filling, etc.
   BrushStyle _brush_style;      // for filling, etc
   Color _pen_color;             // color we draw text, graphics with
   PenStyle _pen_style;          // dashed, solid, etc
   PenThickness _pen_thickness;  // thick or thin
   Color _bg_color;              // background color (usually white)

   
   char _fontbuff[128];   // current font string - DDC 1/13/96
   FontStyle _font_style; // current font style  - DDC 1/13/96
   Bool   _font_loaded;   // tells if a font is loaded or not - DDC 1/15/96
   double _font_size;     // font size in point number - DDC 1/15/96
   double _font_scale;    // font scale factor 1 = 14 point. - DDC 1/17/96
   int _coord_font_size;  // font point for size 1 in current coord system - DDC 3/28/96
   int _large_font_size;  // font size of large font used for scaling - DDC 3/27/96
   int _height_pixels;    // pixel size of font about 12 point - DDC 3/28/96
   int _height_size;      // size of font in case 12 pt isn't there - DDC 3/28/96     

   RopMode _rop_mode;     // current ROP mode - DDC 1/17/96
   XFontStruct* _fontinfo_ptr; 	// points to font info for all fonts used
   Pixmap _fill_stipple;		// for various drawing fill styles
};


#endif
