/****************************************************************************
** COPYRIGHT (C):    1993 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          ChiLib
** FILE:             chiqueue.cpp
** PURPOSE:          Queue template
** VERSION           1.0
** LANGUAGE:         Borland C++ 3.1
** TARGET:           IBM PC
** PROGRAMMER:       Cay Horstmann (CSH)
** START DATE:       7-29-94 (CSH)
** UPDATE HISTORY:
**
****************************************************************************/

/* IMPORT ******************************************************************/

#ifndef EXPORT
#include "chisetup.h"
#endif

EXPORT
#include <assert.h>

EXPORT
#include "chilist.h"

#include "chiqueue.h"

/* TYPES *******************************************************************/

#ifndef EXPORT

template<class T> class Chi_Queue
/* PURPOSE:    A queue
*/
{
public:
   Chi_Queue();
   void insert(T n);
   T remove();
   void empty();

   T head() const;
   int length() const;

   Chi_Queue(const Chi_Queue<T>&);
   const Chi_Queue<T>& operator=(const Chi_Queue<T>&);
   ~Chi_Queue();

private:
   void free();
   void copy(const Chi_Queue<T>&);

   Chi_Link<T>* _head;
   Chi_Link<T>* _tail;
   int _len;

};

//***************************************************************************

template<class T> Chi_Queue<T>::Chi_Queue() :
   _head(0),
   _tail(0),
   _len(0)
{}

//............................................................................

template<class T> Chi_Queue<T>::~Chi_Queue()
{  free();
}

//............................................................................

template<class T> Chi_Queue<T>::Chi_Queue(const Chi_Queue<T>& b)
{  copy(b);
}

//............................................................................

template<class T> const Chi_Queue<T>& Chi_Queue<T>::operator=(const Chi_Queue<T>& b)
{  if (this != &b)
   {  free();
      copy(b);
   }
   return *this;
}

//............................................................................

template<class T> void Chi_Queue<T>::free()
{  Chi_Link<T>* p = _head;
   while(p != 0)
   {  Chi_Link<T>* q = p;
      p = p->_next;
      delete q;
   }
}

//............................................................................

template<class T> void Chi_Queue<T>::copy(const Chi_Queue<T>& b)
{  _len = b._len;
   _head = _tail = 0;

   Chi_Link<T>* pa = 0;
   for (Chi_Link<T>* pb = b._head; pb != 0; pb = pb->_next)
   {  Chi_Link<T>* pnew = new Chi_Link<T>(pb->_info, 0);
      assert(pnew);
      if (pa)
         pa = pa->_next = pnew;
      else
         pa = _head = pnew;
      if (b._tail == pb) _tail = pa;
   }
}

//............................................................................

template<class T> void Chi_Queue<T>::empty()
/* PURPOSE:    remove all elements from the container
*/
{  free();
   _head = 0;
   _tail = 0;
   _len = 0;
}

//............................................................................

template<class T> void Chi_Queue<T>::insert(T n)
/* PURPOSE:    insert a value at the tail of the queue
   RECEIVES:   n - the value to insert
*/
{  Chi_Link<T>* p = new Chi_Link<T>(n, 0);
   assert(p);

   if (!_tail)
      _head = p;
   else
      _tail->_next = p;
   _tail = p;

   _len++;
};

//............................................................................

template<class T> T Chi_Queue<T>::remove()
/* PURPOSE:    remove a value from the head of the queue
   RETURNS:    the removed value
*/
{  if (!_head) return Chi_Link<T>::defval();
   Chi_Link<T>* p = _head;
   _head = p->_next;
   if (_head == 0) _tail = 0;
   T r = p->_info;
   delete p;
   _len--;
   return r;
}

//............................................................................

template<class T> T Chi_Queue<T>::head() const
/* RETURNS:    the value stored at the head of the queue
*/
{  if (!_head) return Chi_Link<T>::defval();
   return _head->_info;
}

//............................................................................

template<class T> int Chi_Queue<T>::length() const
/* RETURNS:    the number of elements in the queue
*/
{  return _len;
}

#endif

/* TEST ********************************************************************/

#ifdef CHIQUEUE_TEST

int main()
{  Chi_Queue<int> q;
   for (int e = 1; e <= 10; e++)
      q.insert(11 * e % 100);
   for (int i = 1; i <= 10; i++)
   {  for (int e = 1; e <= 10; e++)
         q.insert(11 * (10 * i + e) % 100);
      for (int d = 1; d <= 10; d++)
         cout << q.remove() << " ";
      cout << endl;
   }
   for (int d = 1; d <= 10; d++)
      cout << q.remove() << " ";
   CHI_ASSERT(q.length() == 0);
   return 0;
}

#endif

/***************************************************************************/
