#ifndef SIMFRMWK_H
#define SIMFRMWK_H

/****************************************************************************
** COPYRIGHT (C):    1994 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          Mastering Object-Oriented Design
** FILE:             simfrmwk.h
** PURPOSE:          A simulation framework
** VERSION           1.0
** LANGUAGE:         Borland C++ 3.0
** TARGET:           IBM PC
** PROGRAMMER:       Cay Horstmann (CSH)
** START DATE:       9-18-94 (CSH)
** UPDATE HISTORY:
****************************************************************************/

#line 16 "SIMFRMWK.cpp"

#include "chisetup.h"

#line 21 "SIMFRMWK.cpp"

#ifndef CHIPRIOQ_H
#include "chiprioq.h"
#endif

#line 27 "SIMFRMWK.cpp"


class Simulation;

/*-------------------------------------------------------------------------*/

typedef double Time; // measured in minutes

/*-------------------------------------------------------------------------*/

class Event
/* PURPOSE:    An event that is scheduled for execution at some time
*/
{
public:
   Event(Time t);

   virtual void process(Simulation&) = 0;
   Time time() const;
   virtual ~Event() {}
private:
   Time _time;
};

/*-------------------------------------------------------------------------*/

class Simulation
/* PURPOSE:    A generic event-driven simulation
*/
{
public:
   Simulation();
   void run(Time from, Time to);
   void schedule(Event*);
   virtual void initialize() = 0;
   virtual void update() = 0;
   virtual void terminate() = 0;

   Time now() const;

   virtual ~Simulation() {}

private:
   Time _now;
   Chi_PtrPriorityQueue<Event> _event_queue;
};


#line 104 "SIMFRMWK.cpp"
extern double expdist(double mean);

#line 200 "SIMFRMWK.cpp"


inline Time Simulation::now() const
/* RETURNS:    the current simulation time
*/
{  return _now;
}


#endif
